############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2017 Nicolas Agustín Torres <nicolastrres@gmail.com>               #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Huan-Cheng Chang <changhc84@gmail.com>                        #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

from . import Framework


class CommitComment(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.comment = self.g.get_repo("PyGithub/PyGithub").get_comment(1362000)

    def testAttributes(self):
        self.assertEqual(self.comment.author_association, "MEMBER")
        self.assertEqual(self.comment.body, "Comment created by PyGithub\n")
        self.assertEqual(self.comment.commit_id, "6945921c529be14c3a8f566dd1e483674516d46d")
        self.assertEqual(self.comment.created_at, datetime(2012, 5, 22, 18, 49, 34, tzinfo=timezone.utc))
        self.assertEqual(
            self.comment.html_url,
            "https://github.com/PyGithub/PyGithub/commit/6945921c529be14c3a8f566dd1e483674516d46d#commitcomment-1362000",
        )
        self.assertEqual(self.comment.id, 1362000)
        self.assertEqual(self.comment.line, 26)
        self.assertEqual(self.comment.node_id, "MDEzOkNvbW1pdENvbW1lbnQxMzYyMDAw")
        self.assertEqual(self.comment.path, "codegen/templates/GithubObject.MethodBody.UseResult.py")
        self.assertEqual(self.comment.position, None)
        self.assertEqual(
            self.comment.reactions,
            {
                "url": "https://api.github.com/repos/PyGithub/PyGithub/comments/1362000/reactions",
                "total_count": 2,
                "+1": 1,
                "-1": 0,
                "laugh": 0,
                "hooray": 1,
                "confused": 0,
                "heart": 0,
                "rocket": 0,
                "eyes": 0,
            },
        )
        self.assertEqual(self.comment.updated_at, datetime(2012, 5, 22, 18, 49, 34, tzinfo=timezone.utc))
        self.assertEqual(self.comment.url, "https://api.github.com/repos/PyGithub/PyGithub/comments/1362000")
        self.assertEqual(self.comment.user.login, "jacquev6")
        self.assertEqual(repr(self.comment), 'CommitComment(user=NamedUser(login="jacquev6"), id=1362000)')

    def testLazyAttributes(self):
        comment = self.g.withLazy(True).get_repo("lazy/repo").get_comment(42)
        self.assertEqual(str(comment), "CommitComment(user=None, id=42)")
        self.assertEqual(comment.id, 42)
        self.assertEqual(comment.url, "/repos/lazy/repo/comments/42")

    def testEdit(self):
        self.comment.edit("Comment edited by PyGithub")

    def testDelete(self):
        self.comment.delete()

    def testGetReactions(self):
        reactions = self.comment.get_reactions()
        self.assertEqual(reactions[0].content, "+1")

    def testCreateReaction(self):
        reaction = self.comment.create_reaction("hooray")

        self.assertEqual(reaction.id, 17283092)
        self.assertEqual(reaction.content, "hooray")

    def testDeleteReaction(self):
        self.assertTrue(self.comment.delete_reaction(85737646))



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

from . import Framework


class UserKey(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.key = self.g.get_user().get_key(2626650)

    def testAttributes(self):
        self.assertEqual(self.key.created_at, datetime(2024, 12, 23, 12, 34, 56, tzinfo=timezone.utc))
        self.assertEqual(self.key.id, 2626650)
        self.assertEqual(
            self.key.key,
            "ssh-rsa AAAAB3NzaC1yc2EAAAABIwAAAQEA2Mm0RjTNAYFfSCtUpO54usdseroUSIYg5KX4JoseTpqyiB/hqewjYLAdUq/tNIQzrkoEJWSyZrQt0ma7/YCyMYuNGd3DU6q6ZAyBeY3E9RyCiKjO3aTL2VKQGFvBVVmGdxGVSCITRphAcsKc/PF35/fg9XP9S0anMXcEFtdfMHz41SSw+XtE+Vc+6cX9FuI5qUfLGbkv8L1v3g4uw9VXlzq4GfTA+1S7D6mcoGHopAIXFlVr+2RfDKdSURMcB22z41fljO1MW4+zUS/4FyUTpL991es5fcwKXYoiE+x06VJeJJ1Krwx+DZj45uweV6cHXt2JwJEI9fWB6WyBlDejWw==",
        )
        self.assertIsNone(self.key.last_used)
        self.assertTrue(self.key.read_only)
        self.assertEqual(self.key.title, "Key added through PyGithub")
        self.assertEqual(self.key.url, "https://api.github.com/user/keys/2626650")
        self.assertTrue(self.key.verified)
        self.assertEqual(
            repr(self.key),
            'UserKey(title="Key added through PyGithub", id=2626650)',
        )

    def testDelete(self):
        self.key.delete()



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

import os
from datetime import datetime, timezone
from tempfile import NamedTemporaryFile
from unittest import skipIf

from . import Framework


class GitReleaseAsset(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.release = self.g.get_repo("EnricoMi/PyGithub", lazy=True).get_release(197548596)
        self.asset = self.release.assets[0]

    def testAttributes(self):
        self.assertEqual(
            self.asset.browser_download_url, "https://github.com/EnricoMi/PyGithub/releases/download/v1.55/asset1.md"
        )
        self.assertEqual(self.asset.content_type, "text/markdown")
        self.assertEqual(self.asset.created_at, datetime(2025, 1, 30, 11, 11, 32, tzinfo=timezone.utc))
        self.assertEqual(self.asset.digest, "sha256:abc123")
        self.assertEqual(self.asset.download_count, 0)
        self.assertEqual(self.asset.id, 224868540)
        self.assertIsNone(self.asset.label)
        self.assertEqual(self.asset.name, "asset1.md")
        self.assertEqual(self.asset.node_id, "RA_kwDOGpsAJ84NZzi8")
        self.assertEqual(self.asset.size, 2524)
        self.assertEqual(self.asset.state, "uploaded")
        self.assertEqual(self.asset.updated_at, datetime(2025, 1, 30, 11, 12, tzinfo=timezone.utc))
        self.assertEqual(self.asset.uploader.login, "EnricoMi")
        self.assertEqual(self.asset.url, "https://api.github.com/repos/EnricoMi/PyGithub/releases/assets/224868540")

    def testLazyAttributes(self):
        asset = self.g.withLazy(True).get_repo("lazy/repo").get_release_asset(42)
        self.assertEqual(str(asset), 'GitReleaseAsset(url="/repos/lazy/repo/releases/assets/42")')
        self.assertEqual(asset.id, 42)
        self.assertEqual(asset.url, "/repos/lazy/repo/releases/assets/42")

    @skipIf(os.name == "nt", "not working on Windows")
    def testDownloadAssetFile(self):
        with NamedTemporaryFile(mode="rb") as file:
            self.asset.download_asset(file.name)
            content = file.read()
        self.assertEqual(len(content), 2524)
        self.assertEqual(
            content[:172],
            b"# PyGitHub\n\n"
            b"[![PyPI](https://img.shields.io/pypi/v/PyGithub.svg)](https://pypi.python.org/pypi/PyGithub)\n"
            b"![CI](https://github.com/PyGithub/PyGithub/workflows/CI/badge.svg)\n",
        )
        self.assertEqual(content[-50:], b"send an email to someone in the MAINTAINERS file.\n")

        source_asset = self.release.assets[1]
        with NamedTemporaryFile(mode="rb") as file:
            source_asset.download_asset(file.name)
            content = file.read()
        self.assertEqual(len(content), 1199)
        self.assertEqual(content[:19], b"\x1f\x8b\x08\x08\xf5\x9aag\x00\x03README.md")

    def testDownloadAssetStream(self):
        (_, _, chunks) = self.asset.download_asset()
        content = b"".join([chunk for chunk in chunks if chunk])
        self.assertEqual(len(content), 2524)
        self.assertEqual(
            content[:172],
            b"# PyGitHub\n\n"
            b"[![PyPI](https://img.shields.io/pypi/v/PyGithub.svg)](https://pypi.python.org/pypi/PyGithub)\n"
            b"![CI](https://github.com/PyGithub/PyGithub/workflows/CI/badge.svg)\n",
        )
        self.assertEqual(content[-50:], b"send an email to someone in the MAINTAINERS file.\n")

        source_asset = self.release.assets[1]
        (_, _, chunks) = source_asset.download_asset()
        content = b"".join([chunk for chunk in chunks if chunk])
        self.assertEqual(len(content), 1199)
        self.assertEqual(content[:19], b"\x1f\x8b\x08\x08\xf5\x9aag\x00\x03README.md")



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2017 Simon <spam@esemi.ru>                                         #
# Copyright 2018 Bruce Richardson <itsbruce@workshy.org>                       #
# Copyright 2018 Riccardo Pittau <elfosardo@users.noreply.github.com>          #
# Copyright 2018 namc <namratachaudhary@users.noreply.github.com>              #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 Surya Teja <94suryateja@gmail.com>                            #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Daniel Haas <thisisdhaas@gmail.com>                           #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone
from unittest import mock

import github.NamedUser
import github.Requester
from github.GithubObject import is_undefined

from . import Framework


class NamedUser(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.user = self.g.get_user("jacquev6")

    def testAttributes(self):
        self.assertEqual(self.user.avatar_url, "https://avatars.githubusercontent.com/u/327146?v=4")
        self.assertIsNone(self.user.bio)
        self.assertEqual(self.user.blog, "http://vincent-jacques.net")
        self.assertIsNone(self.user.business_plus)
        self.assertIsNone(self.user.collaborators)
        self.assertIsNone(self.user.company)
        self.assertIsNone(self.user.contributions)
        self.assertEqual(self.user.created_at, datetime(2010, 7, 9, 6, 10, 6, tzinfo=timezone.utc))
        self.assertIsNone(self.user.disk_usage)
        self.assertIsNone(self.user.display_login)
        self.assertEqual(self.user.email, "vincent@vincent-jacques.net")
        self.assertEqual(self.user.events_url, "https://api.github.com/users/jacquev6/events{/privacy}")
        self.assertEqual(self.user.followers, 98)
        self.assertEqual(self.user.followers_url, "https://api.github.com/users/jacquev6/followers")
        self.assertEqual(self.user.following, 62)
        self.assertEqual(self.user.following_url, "https://api.github.com/users/jacquev6/following{/other_user}")
        self.assertEqual(self.user.gists_url, "https://api.github.com/users/jacquev6/gists{/gist_id}")
        self.assertEqual(self.user.gravatar_id, "")
        self.assertTrue(self.user.hireable)
        self.assertEqual(self.user.html_url, "https://github.com/jacquev6")
        self.assertEqual(self.user.id, 327146)
        self.assertIsNone(self.user.ldap_dn)
        self.assertEqual(self.user.location, "France")
        self.assertEqual(self.user.login, "jacquev6")
        self.assertEqual(self.user.name, "Vincent Jacques")
        self.assertEqual(self.user.node_id, "MDQ6VXNlcjMyNzE0Ng==")
        self.assertIsNone(self.user.notification_email)
        self.assertEqual(self.user.organizations_url, "https://api.github.com/users/jacquev6/orgs")
        self.assertIsNone(self.user.owned_private_repos)
        self.assertIsNone(self.user.permissions)
        self.assertIsNone(self.user.plan)
        self.assertIsNone(self.user.plan)
        self.assertIsNone(self.user.plan)
        self.assertIsNone(self.user.plan)
        self.assertIsNone(self.user.private_gists)
        self.assertEqual(self.user.public_gists, 18)
        self.assertEqual(self.user.public_repos, 38)
        self.assertEqual(self.user.received_events_url, "https://api.github.com/users/jacquev6/received_events")
        self.assertEqual(self.user.repos_url, "https://api.github.com/users/jacquev6/repos")
        self.assertIsNone(self.user.role_name)
        self.assertEqual(self.user.site_admin, False)
        self.assertIsNone(self.user.starred_at)
        self.assertEqual(self.user.starred_url, "https://api.github.com/users/jacquev6/starred{/owner}{/repo}")
        self.assertEqual(self.user.subscriptions_url, "https://api.github.com/users/jacquev6/subscriptions")
        self.assertIsNone(self.user.suspended_at)
        self.assertIsNone(self.user.text_matches)
        self.assertIsNone(self.user.total_private_repos)
        self.assertIsNone(self.user.twitter_username)
        self.assertIsNone(self.user.two_factor_authentication)
        self.assertEqual(self.user.type, "User")
        self.assertEqual(self.user.updated_at, datetime(2024, 10, 20, 7, 14, 52, tzinfo=timezone.utc))
        self.assertEqual(self.user.url, "https://api.github.com/users/jacquev6")
        self.assertEqual(self.user.node_id, "MDQ6VXNlcjMyNzE0Ng==")
        self.assertEqual(repr(self.user), 'NamedUser(login="jacquev6")')
        self.assertEqual(repr(self.user.plan), "None")
        self.assertEqual(self.user.user_view_type, "public")

    def testLazyAttributes(self):
        user = self.g.withLazy(True).get_user("lazyUser", lazy=True)
        self.assertEqual(str(user), 'NamedUser(login="lazyUser")')
        self.assertEqual(user._identity, "lazyUser")
        self.assertEqual(user.login, "lazyUser")
        self.assertEqual(user.url, "/users/lazyUser")

    def testCreatFromUrl(self):
        requester = mock.Mock(github.Requester.Requester, base_url="https://test.ing/api/", is_not_lazy=False)

        for base_url in [requester.base_url[:-1], ""]:
            user = github.NamedUser.NamedUser(requester, url=f"{base_url}/user/12345")
            self.assertEqual(user.url, f"{base_url}/user/12345", msg=f"base url: '{base_url}'")
            self.assertEqual(user.id, 12345)
            self.assertTrue(is_undefined(user._login))

            user = github.NamedUser.NamedUser(requester, url=f"{base_url}/users/login")
            self.assertEqual(user.url, f"{base_url}/users/login", msg=f"base url: '{base_url}'")
            self.assertTrue(is_undefined(user._id))
            self.assertEqual(user.login, "login")

            user = github.NamedUser.NamedUser(requester, url=f"{base_url}/users/12345")
            self.assertEqual(user.url, f"{base_url}/users/12345", msg=f"base url: '{base_url}'")
            self.assertTrue(is_undefined(user._id))
            self.assertEqual(user.login, "12345")

    def testCreatFromAttributes(self):
        requester = mock.Mock(github.Requester.Requester, base_url="https://test.ing/api/", is_not_lazy=False)

        user = github.NamedUser.NamedUser(requester, attributes={"id": 12345})
        self.assertEqual(user.url, "/user/12345")
        self.assertEqual(user.id, 12345)
        self.assertTrue(is_undefined(user._name))

        user = github.NamedUser.NamedUser(requester, attributes={"login": "login"})
        self.assertEqual(user.url, "/users/login")
        self.assertTrue(is_undefined(user._id))
        self.assertEqual(user.login, "login")

    def testAttributesOfOtherUser(self):
        self.user = self.g.get_user("nvie")
        self.assertEqual(
            self.user.avatar_url,
            "https://secure.gravatar.com/avatar/c5a7f21b46df698f3db31c37ed0cf55a?d=https://a248.e.akamai.net/assets.github.com%2Fimages%2Fgravatars%2Fgravatar-140.png",
        )
        self.assertEqual(self.user.bio, None)
        self.assertEqual(self.user.blog, "http://nvie.com")
        self.assertEqual(self.user.collaborators, None)
        self.assertEqual(self.user.company, "3rd Cloud")
        self.assertEqual(
            self.user.created_at,
            datetime(2009, 5, 12, 21, 19, 38, tzinfo=timezone.utc),
        )
        self.assertEqual(self.user.disk_usage, None)
        self.assertEqual(self.user.email, "vincent@3rdcloud.com")
        self.assertEqual(self.user.followers, 296)
        self.assertEqual(self.user.following, 41)
        self.assertEqual(self.user.gravatar_id, "c5a7f21b46df698f3db31c37ed0cf55a")
        self.assertFalse(self.user.hireable)
        self.assertEqual(self.user.html_url, "https://github.com/nvie")
        self.assertEqual(self.user.id, 83844)
        self.assertEqual(self.user.location, "Netherlands")
        self.assertEqual(self.user.login, "nvie")
        self.assertEqual(self.user.name, "Vincent Driessen")
        self.assertEqual(self.user.owned_private_repos, None)
        self.assertEqual(self.user.plan, None)
        self.assertEqual(self.user.private_gists, None)
        self.assertEqual(self.user.public_gists, 16)
        self.assertEqual(self.user.public_repos, 61)
        self.assertEqual(self.user.suspended_at, None)
        self.assertEqual(self.user.total_private_repos, None)
        self.assertEqual(self.user.twitter_username, "nvie")
        self.assertEqual(self.user.type, "User")
        self.assertEqual(self.user.url, "https://api.github.com/users/nvie")
        self.assertEqual(self.user.node_id, "MDQ6VXNlcjgzODQ0")
        self.assertEqual(repr(self.user), 'NamedUser(login="nvie")')

    def testGetGists(self):
        self.assertListKeyEqual(
            self.user.get_gists(),
            lambda g: g.description,
            [
                "Gist created by PyGithub",
                "FairThreadPoolPool.cpp",
                "How to error 500 Github API v3, as requested by Rick (GitHub Staff)",
                "Cadfael: order of episodes in French DVD edition",
            ],
        )
        self.assertListKeyEqual(
            self.user.get_gists(since=datetime(2012, 3, 1, 17, 0, 0)),
            lambda g: g.description,
            ["Gist created by PyGithub", "FairThreadPoolPool.cpp"],
        )

    def testGetFollowers(self):
        self.assertListKeyEqual(
            self.user.get_followers(),
            lambda f: f.login,
            [
                "jnorthrup",
                "brugidou",
                "regisb",
                "walidk",
                "afzalkhan",
                "sdanzan",
                "vineus",
                "gturri",
                "fjardon",
                "cjuniet",
                "jardon-u",
                "kamaradclimber",
                "L42y",
            ],
        )

    def testGetFollowing(self):
        self.assertListKeyEqual(
            self.user.get_following(),
            lambda f: f.login,
            [
                "nvie",
                "schacon",
                "jamis",
                "chad",
                "unclebob",
                "dabrahams",
                "jnorthrup",
                "brugidou",
                "regisb",
                "walidk",
                "tanzilli",
                "fjardon",
                "r3c",
                "sdanzan",
                "vineus",
                "cjuniet",
                "gturri",
                "ant9000",
                "asquini",
                "claudyus",
                "jardon-u",
                "s-bernard",
                "kamaradclimber",
                "Lyloa",
            ],
        )

    def testHasInFollowing(self):
        nvie = self.g.get_user("nvie")
        self.assertTrue(self.user.has_in_following(nvie))

    def testGetOrgs(self):
        self.assertListKeyEqual(self.user.get_orgs(), lambda o: o.login, ["BeaverSoftware"])

    def testGetOrganizationMembership(self):
        o = self.user.get_orgs()
        membership = self.user.get_organization_membership(o[0])
        self.assertEqual(
            repr(membership),
            'Membership(url="https://api.github.com/orgs/BeaverSoftware/memberships/jacquev6")',
        )
        self.assertEqual(self.user.login, membership.user.login)
        self.assertEqual(membership.state, "active")
        self.assertEqual(membership.role, "member")
        self.assertEqual(
            membership.url,
            "https://api.github.com/orgs/BeaverSoftware/memberships/jacquev6",
        )
        self.assertEqual(membership.organization.login, "BeaverSoftware")
        self.assertEqual(membership.organization_url, "https://api.github.com/orgs/BeaverSoftware")

    def testGetOrganizationMembershipNotMember(self):
        from github import UnknownObjectException

        self.assertRaises(
            UnknownObjectException,
            self.user.get_organization_membership,
            "BeaverSoftware",
        )

    def testGetRepo(self):
        self.assertEqual(
            self.user.get_repo("PyGithub").description,
            "Python library implementing the full Github API v3",
        )

    def testGetRepos(self):
        self.assertListKeyEqual(
            self.user.get_repos(),
            lambda r: r.name,
            [
                "TestPyGithub",
                "django",
                "PyGithub",
                "developer.github.com",
                "acme-public-website",
                "C4Planner",
                "DrawTurksHead",
                "DrawSyntax",
                "QuadProgMm",
                "Boost.HierarchicalEnum",
                "ViDE",
            ],
        )

    def testGetReposWithAllArgs(self):
        self.assertListKeyEqual(
            self.user.get_repos(type="owner", sort="created", direction="asc"),
            lambda r: r.name,
            [
                "DrawTurksHead",
                "vincent-jacques.net",
                "IpMap",
                "MockMockMock",
                "ActionTree",
                "InteractiveCommandLine",
                "RecursiveDocument",
                "MarblesCollide",
                "jacquev6.github.io",
                "LowVoltage",
            ],
        )

    def testGetWatched(self):
        self.assertListKeyEqual(
            self.user.get_watched(),
            lambda r: r.name,
            [
                "git",
                "boost.php",
                "capistrano",
                "boost.perl",
                "git-subtree",
                "git-hg",
                "homebrew",
                "celtic_knot",
                "twisted-intro",
                "markup",
                "hub",
                "gitflow",
                "murder",
                "boto",
                "agit",
                "d3",
                "pygit2",
                "git-pulls",
                "django_mathlatex",
                "scrumblr",
                "developer.github.com",
                "python-github3",
                "PlantUML",
                "bootstrap",
                "drawnby",
                "django-socketio",
                "django-realtime",
                "playground",
                "BozoCrack",
                "FatherBeaver",
                "PyGithub",
                "django",
                "django",
                "TestPyGithub",
            ],
        )

    def testGetStarred(self):
        self.assertListKeyEqual(
            self.user.get_starred(),
            lambda r: r.name,
            [
                "git",
                "boost.php",
                "capistrano",
                "boost.perl",
                "git-subtree",
                "git-hg",
                "homebrew",
                "celtic_knot",
                "twisted-intro",
                "markup",
                "hub",
                "gitflow",
                "murder",
                "boto",
                "agit",
                "d3",
                "pygit2",
                "git-pulls",
                "django_mathlatex",
                "scrumblr",
                "developer.github.com",
                "python-github3",
                "PlantUML",
                "bootstrap",
                "drawnby",
                "django-socketio",
                "django-realtime",
                "playground",
                "BozoCrack",
                "FatherBeaver",
                "amaunet",
                "django",
                "django",
                "moviePlanning",
                "folly",
            ],
        )

    def testGetSubscriptions(self):
        self.assertListKeyEqual(
            self.user.get_subscriptions(),
            lambda r: r.name,
            [
                "ViDE",
                "Boost.HierarchicalEnum",
                "QuadProgMm",
                "DrawSyntax",
                "DrawTurksHead",
                "PrivateStuff",
                "vincent-jacques.net",
                "Hacking",
                "C4Planner",
                "developer.github.com",
                "PyGithub",
                "PyGithub",
                "django",
                "CinePlanning",
                "PyGithub",
                "PyGithub",
                "PyGithub",
                "IpMap",
                "PyGithub",
                "PyGithub",
                "PyGithub",
                "PyGithub",
                "PyGithub",
                "PyGithub",
                "PyGithub",
                "PyGithub",
                "PyGithub",
                "PyGithub",
                "PyGithub",
                "PyGithub",
            ],
        )

    def testGetEvents(self):
        self.assertListKeyBegin(
            self.user.get_events(),
            lambda e: e.type,
            ["GistEvent", "IssueCommentEvent", "PushEvent", "IssuesEvent"],
        )

    def testGetPublicEvents(self):
        self.assertListKeyBegin(
            self.user.get_public_events(),
            lambda e: e.type,
            ["PushEvent", "CreateEvent", "GistEvent", "IssuesEvent"],
        )

    def testGetPublicReceivedEvents(self):
        self.assertListKeyBegin(
            self.user.get_public_received_events(),
            lambda e: e.type,
            [
                "IssueCommentEvent",
                "IssueCommentEvent",
                "IssueCommentEvent",
                "IssueCommentEvent",
            ],
        )

    def testGetReceivedEvents(self):
        self.assertListKeyBegin(
            self.user.get_received_events(),
            lambda e: e.type,
            [
                "IssueCommentEvent",
                "IssueCommentEvent",
                "IssueCommentEvent",
                "IssueCommentEvent",
            ],
        )

    def testGetKeys(self):
        self.assertListKeyEqual(
            self.user.get_keys(),
            lambda k: k.id,
            [3557894, 3791954, 3937333, 4051357, 4051492],
        )

    def testUserEquality(self):
        u1 = self.g.get_user("nvie")
        u2 = self.g.get_user("nvie")
        self.assertTrue(u1 == u2)
        self.assertEqual(u1, u2)
        self.assertEqual(u1.__hash__(), u2.__hash__())


class OrganizationInvitation(Framework.TestCase):
    def setUp(self):
        super().setUp()
        # TODO: create an instance of type OrganizationInvitation and assign to self.attr, then run:
        #   pytest tests/OrganizationInvitation.py -k testAttributes --record
        #   ./scripts/update-assertions.sh tests/OrganizationInvitation.py testAttributes
        self.org = self.g.get_organization("TestOrganization2072")
        self.invitations = list(self.org.invitations())
        self.assertGreater(len(self.invitations), 0)
        self.invitation = self.invitations[0]

    def testAttributes(self):
        self.assertIsNotNone(self.invitation)
        self.assertEqual(self.invitation.created_at, datetime(2021, 10, 12, 13, 32, 33, tzinfo=timezone.utc))
        self.assertEqual(self.invitation.email, "foo@bar.org")
        self.assertIsNone(self.invitation.failed_at)
        self.assertIsNone(self.invitation.failed_reason)
        self.assertEqual(self.invitation.id, 28984230)
        self.assertIsNone(self.invitation.invitation_source)
        self.assertEqual(
            self.invitation.invitation_teams_url,
            "https://api.github.com/organizations/92288976/invitations/28984230/teams",
        )
        self.assertEqual(self.invitation.inviter.login, "jsimpso")
        self.assertIsNone(self.invitation.login)
        self.assertEqual(self.invitation.node_id, "OI_kwDOBYA30M4BukOm")
        self.assertEqual(self.invitation.role, "direct_member")
        self.assertEqual(self.invitation.team_count, 0)

    def testCancel(self):
        self.assertFalse(any([i for i in self.org.invitations() if i.email == "foo@bar.org"]))
        self.org.invite_user(email="foo@bar.org")
        self.assertTrue(any([i for i in self.org.invitations() if i.email == "foo@bar.org"]))
        invitation = [i for i in self.org.invitations() if i.email == "foo@bar.org"][0]
        self.assertTrue(self.org.cancel_invitation(invitation))
        # copy replay data of self.org.cancel_invitation(invitation) call, fix HTTP path
        self.assertTrue(invitation.cancel())



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Peter Golm <golm.peter@gmail.com>                             #
# Copyright 2013 Steve Brown <steve@evolvedlight.co.uk>                        #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Tyler Treat <ttreat31@gmail.com>                              #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 Bruce Richardson <itsbruce@workshy.org>                       #
# Copyright 2018 Svend Sorensen <svend@svends.net>                             #
# Copyright 2018 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Nikolay Edigaryev <edigaryev@gmail.com>                       #
# Copyright 2020 Omar Brikaa <brikaaomar@gmail.com>                            #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2023 Jonathan Greg <31892308+jmgreg31@users.noreply.github.com>    #
# Copyright 2023 Joseph Henrich <crimsonknave@gmail.com>                       #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2024 Min RK <benjaminrk@gmail.com>                                 #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from datetime import datetime, timezone

import github

from . import Framework


class Github(Framework.TestCase):
    def testGetGists(self):
        self.assertListKeyBegin(
            self.g.get_gists(),
            lambda g: g.id,
            [
                "2729695",
                "2729656",
                "2729597",
                "2729584",
                "2729569",
                "2729554",
                "2729543",
                "2729537",
                "2729536",
                "2729533",
                "2729525",
                "2729522",
                "2729519",
                "2729515",
                "2729506",
                "2729487",
                "2729484",
                "2729482",
                "2729441",
                "2729432",
                "2729420",
                "2729398",
                "2729372",
                "2729371",
                "2729351",
                "2729346",
                "2729316",
                "2729304",
                "2729296",
                "2729276",
                "2729272",
                "2729265",
                "2729195",
                "2729160",
                "2729143",
                "2729127",
                "2729119",
                "2729113",
                "2729103",
                "2729069",
                "2729059",
                "2729051",
                "2729029",
                "2729027",
                "2729026",
                "2729022",
                "2729002",
                "2728985",
                "2728979",
                "2728964",
                "2728937",
                "2728933",
                "2728884",
                "2728869",
                "2728866",
                "2728855",
                "2728854",
                "2728853",
                "2728846",
                "2728825",
                "2728814",
                "2728813",
                "2728812",
                "2728805",
                "2728802",
                "2728800",
                "2728798",
                "2728797",
                "2728796",
                "2728793",
                "2728758",
                "2728754",
                "2728751",
                "2728748",
                "2728721",
                "2728716",
                "2728715",
                "2728705",
                "2728701",
                "2728699",
                "2728697",
                "2728688",
                "2728683",
                "2728677",
                "2728649",
                "2728640",
                "2728625",
                "2728620",
                "2728615",
                "2728614",
                "2728565",
                "2728564",
                "2728554",
                "2728523",
                "2728519",
                "2728511",
                "2728497",
                "2728496",
                "2728495",
                "2728487",
            ],
        )

    def testGetGistsWithSince(self):
        self.assertListKeyBegin(
            self.g.get_gists(since=datetime(2018, 10, 2, 10, 38, 30, 00)),
            lambda g: g.id,
            [
                "69b8a5831b74946db944c5451017fa40",
                "c22050a8705e93d170e0d4ca9c02e40c",
                "a7a95e1a194e07960364a5b32c56ac5f",
                "a25d9ace89b574f95bf0724f95a84fc2",
                "3195465",
            ],
        )

    def testGetGlobalAdvisories(self):
        self.assertListKeyEqual(
            self.g.get_global_advisories(ecosystem="pub"),
            lambda a: a.ghsa_id,
            [
                "GHSA-9324-jv53-9cc8",
                "GHSA-9f2c-xxfm-32mj",
                "GHSA-4xh4-v2pq-jvhm",
                "GHSA-jwpw-q68h-r678",
                "GHSA-4rgh-jx4f-qfcq",
            ],
        )

    def testGetGlobalAdvisoriesByGHSA(self):
        self.assertListKeyEqual(
            self.g.get_global_advisories(ghsa_id="GHSA-9324-jv53-9cc8"),
            lambda a: a.ghsa_id,
            [
                "GHSA-9324-jv53-9cc8",
            ],
        )

    def testGetGlobalAdvisoriesByCVE(self):
        self.assertListKeyEqual(
            self.g.get_global_advisories(cve_id="CVE-2023-38503"),
            lambda a: a.ghsa_id,
            [
                "GHSA-gggm-66rh-pp98",
            ],
        )

    def testGetGlobalAdvisoriesManyFilters(self):
        cases = [
            {"cwes": [200, 900], "affects": ["directus", "made_up"], "modified": ">2023-07-01"},
            {"cwes": ["200", "900"], "affects": ["directus"], "updated": ">2023-07-01"},
            {"cwes": "200,900", "affects": "directus", "published": ">2023-07-01"},
        ]
        for case in cases:
            with self.subTest(**case):
                advisories = self.g.get_global_advisories(
                    type="reviewed",
                    ecosystem="npm",
                    severity="medium",
                    #  cwes=case["cwes"],
                    is_withdrawn=False,
                    #  affects=case["affects"],
                    #  modified=">2023-07-01",
                    direction="desc",
                    sort="updated",
                    **case,
                )
                self.assertListKeyEqual(
                    advisories,
                    lambda a: a.ghsa_id,
                    [
                        "GHSA-gggm-66rh-pp98",
                    ],
                )

    def testGetHooks(self):
        hooks = self.g.get_hooks()
        hook = hooks[0]
        self.assertEqual(hook.name, "activecollab")
        self.assertEqual(hook.supported_events, ["push"])
        self.assertEqual(hook.events, ["push"])
        self.assertEqual(
            hook.schema,
            [
                ["string", "url"],
                ["string", "token"],
                ["string", "project_id"],
                ["string", "milestone_id"],
                ["string", "category_id"],
            ],
        )

    def testGetEmojis(self):
        emojis = self.g.get_emojis()
        first = emojis.get("+1")
        self.assertEqual(first, "https://github.global.ssl.fastly.net/images/icons/emoji/+1.png?v5")

    def testGetHook(self):
        hook = self.g.get_hook("activecollab")
        self.assertEqual(hook.name, "activecollab")
        self.assertEqual(hook.supported_events, ["push"])
        self.assertEqual(hook.events, ["push"])
        self.assertEqual(
            hook.schema,
            [
                ["string", "url"],
                ["string", "token"],
                ["string", "project_id"],
                ["string", "milestone_id"],
                ["string", "category_id"],
            ],
        )
        self.assertEqual(repr(hook), 'HookDescription(name="activecollab")')

    def testGetHookDelivery(self):
        delivery = self.g.get_hook_delivery(257993, 12345)
        self.assertEqual(delivery.id, 12345)
        self.assertEqual(delivery.guid, "abcde-12345")
        self.assertEqual(
            delivery.delivered_at,
            datetime(2012, 5, 27, 6, 0, 32, tzinfo=timezone.utc),
        )
        self.assertEqual(delivery.redelivery, False)
        self.assertEqual(delivery.duration, 0.27)
        self.assertEqual(delivery.status, "OK")
        self.assertEqual(delivery.status_code, 200)
        self.assertEqual(delivery.throttled_at, datetime(2024, 1, 2, 12, 34, 56, tzinfo=timezone.utc))
        self.assertEqual(delivery.event, "issues")
        self.assertEqual(delivery.action, "opened")
        self.assertEqual(delivery.installation_id, 123)
        self.assertEqual(delivery.repository_id, 456)
        self.assertEqual(delivery.url, "https://www.example-webhook.com")
        self.assertIsInstance(delivery.request, github.HookDelivery.HookDeliveryRequest)
        self.assertEqual(delivery.request.headers, {"content-type": "application/json"})
        self.assertEqual(delivery.request.payload, {"action": "opened"})
        self.assertIsInstance(delivery.response, github.HookDelivery.HookDeliveryResponse)
        self.assertEqual(delivery.response.headers, {"content-type": "text/html;charset=utf-8"})
        self.assertEqual(delivery.response.payload, "ok")

    def testGetHookDeliveries(self):
        deliveries = list(self.g.get_hook_deliveries(257993))
        self.assertEqual(len(deliveries), 1)
        self.assertEqual(deliveries[0].id, 12345)
        self.assertEqual(deliveries[0].guid, "abcde-12345")
        self.assertEqual(
            deliveries[0].delivered_at,
            datetime(2012, 5, 27, 6, 0, 32, tzinfo=timezone.utc),
        )
        self.assertEqual(deliveries[0].redelivery, False)
        self.assertEqual(deliveries[0].duration, 0.27)
        self.assertEqual(deliveries[0].status, "OK")
        self.assertEqual(deliveries[0].status_code, 200)
        self.assertEqual(deliveries[0].throttled_at, datetime(2024, 1, 2, 12, 34, 56, tzinfo=timezone.utc))
        self.assertEqual(deliveries[0].event, "issues")
        self.assertEqual(deliveries[0].action, "opened")
        self.assertEqual(deliveries[0].installation_id, 123)
        self.assertEqual(deliveries[0].repository_id, 456)
        self.assertEqual(deliveries[0].url, "https://www.example-webhook.com")

    def testGetRepoFromFullName(self):
        self.assertEqual(
            self.g.get_repo("jacquev6/PyGithub").description,
            "Python library implementing the full Github API v3",
        )

    def testGetRepoFromId(self):
        self.assertEqual(
            self.g.get_repo(3544490).description,
            "Python library implementing the full Github API v3",
        )

    def testGetGitignoreTemplates(self):
        self.assertEqual(
            self.g.get_gitignore_templates(),
            [
                "Actionscript",
                "Android",
                "AppceleratorTitanium",
                "Autotools",
                "Bancha",
                "C",
                "C++",
                "CFWheels",
                "CMake",
                "CSharp",
                "CakePHP",
                "Clojure",
                "CodeIgniter",
                "Compass",
                "Concrete5",
                "Coq",
                "Delphi",
                "Django",
                "Drupal",
                "Erlang",
                "ExpressionEngine",
                "Finale",
                "ForceDotCom",
                "FuelPHP",
                "GWT",
                "Go",
                "Grails",
                "Haskell",
                "Java",
                "Jboss",
                "Jekyll",
                "Joomla",
                "Jython",
                "Kohana",
                "LaTeX",
                "Leiningen",
                "LemonStand",
                "Lilypond",
                "Lithium",
                "Magento",
                "Maven",
                "Node",
                "OCaml",
                "Objective-C",
                "Opa",
                "OracleForms",
                "Perl",
                "PlayFramework",
                "Python",
                "Qooxdoo",
                "Qt",
                "R",
                "Rails",
                "RhodesRhomobile",
                "Ruby",
                "Scala",
                "Sdcc",
                "SeamGen",
                "SketchUp",
                "SugarCRM",
                "Symfony",
                "Symfony2",
                "SymphonyCMS",
                "Target3001",
                "Tasm",
                "Textpattern",
                "TurboGears2",
                "Unity",
                "VB.Net",
                "Waf",
                "Wordpress",
                "Yii",
                "ZendFramework",
                "gcov",
                "nanoc",
                "opencart",
            ],
        )

    def testGetGitignoreTemplate(self):
        t = self.g.get_gitignore_template("Python")
        self.assertEqual(t.name, "Python")
        self.assertEqual(
            t.source,
            "*.py[cod]\n\n# C extensions\n*.so\n\n# Packages\n*.egg\n*.egg-info\ndist\nbuild\neggs\nparts\nbin\nvar\nsdist\ndevelop-eggs\n.installed.cfg\nlib\nlib64\n\n# Installer logs\npip-log.txt\n\n# Unit test / coverage reports\n.coverage\n.tox\nnosetests.xml\n\n# Translations\n*.mo\n\n# Mr Developer\n.mr.developer.cfg\n.project\n.pydevproject\n",
        )
        self.assertEqual(repr(t), 'GitignoreTemplate(name="Python")')

        t = self.g.get_gitignore_template("C++")
        self.assertEqual(t.name, "C++")
        self.assertEqual(
            t.source,
            "# Compiled Object files\n*.slo\n*.lo\n*.o\n\n# Compiled Dynamic libraries\n*.so\n*.dylib\n\n# Compiled Static libraries\n*.lai\n*.la\n*.a\n",
        )

    def testStringOfNotSet(self):
        self.assertEqual(str(github.GithubObject.NotSet), "NotSet")

    def testGetUserById(self):
        self.assertEqual(self.g.get_user_by_id(583231).login, "octocat")

    def testGetUsers(self):
        self.assertListKeyBegin(
            self.g.get_users(),
            lambda u: u.login,
            [
                "mojombo",
                "defunkt",
                "pjhyett",
                "wycats",
                "ezmobius",
                "ivey",
                "evanphx",
                "vanpelt",
                "wayneeseguin",
                "brynary",
                "kevinclark",
                "technoweenie",
                "macournoyer",
                "takeo",
                "Caged",
                "topfunky",
                "anotherjesse",
                "roland",
                "lukas",
                "fanvsfan",
                "tomtt",
                "railsjitsu",
                "nitay",
                "kevwil",
                "KirinDave",
                "jamesgolick",
                "atmos",
                "errfree",
                "mojodna",
                "bmizerany",
                "jnewland",
                "joshknowles",
                "hornbeck",
                "jwhitmire",
                "elbowdonkey",
                "reinh",
                "timocratic",
                "bs",
                "rsanheim",
                "schacon",
                "uggedal",
                "bruce",
                "sam",
                "mmower",
                "abhay",
                "rabble",
                "benburkert",
                "indirect",
                "fearoffish",
                "ry",
                "engineyard",
                "jsierles",
                "tweibley",
                "peimei",
                "brixen",
                "tmornini",
                "outerim",
                "daksis",
                "sr",
                "lifo",
                "rsl",
                "imownbey",
                "dylanegan",
                "jm",
                "willcodeforfoo",
                "jvantuyl",
                "BrianTheCoder",
                "freeformz",
                "hassox",
                "automatthew",
                "queso",
                "lancecarlson",
                "drnic",
                "lukesutton",
                "danwrong",
                "hcatlin",
                "jfrost",
                "mattetti",
                "ctennis",
                "lawrencepit",
                "marcjeanson",
                "grempe",
                "peterc",
                "ministrycentered",
                "afarnham",
                "up_the_irons",
                "evilchelu",
                "heavysixer",
                "brosner",
                "danielmorrison",
                "danielharan",
                "kvnsmth",
                "collectiveidea",
                "canadaduane",
                "nate",
                "dstrelau",
                "sunny",
                "dkubb",
                "jnicklas",
                "richcollins",
                "simonjefford",
            ],
        )

    def testGetUsersSince(self):
        self.assertListKeyBegin(self.g.get_users(since=1000), lambda u: u.login, ["sbecker"])

    def testGetOrganizations(self):
        self.assertListKeyBegin(
            self.g.get_organizations(),
            lambda u: u.login,
            [
                "errfree",
                "engineyard",
                "ministrycentered",
                "collectiveidea",
                "ogc",
                "sevenwire",
                "entryway",
            ],
        )

    def testGetOrganizationsSince(self):
        self.assertListKeyBegin(
            self.g.get_organizations(since=1000),
            lambda u: u.login,
            [
                "railslove",
                "railsdog",
                "netguru",
                "webhostio",
                "animikii",
                "sauspiel",
                "wherecloud",
                "triveos",
            ],
        )

    def testGetRepos(self):
        self.assertListKeyBegin(
            self.g.get_repos(),
            lambda r: r.name,
            [
                "grit",
                "merb-core",
                "rubinius",
                "god",
                "jsawesome",
                "jspec",
                "exception_logger",
                "ambition",
            ],
        )

    def testGetReposSince(self):
        self.assertListKeyBegin(
            self.g.get_repos(since=1000),
            lambda r: r.name,
            [
                "jquery-humanize-messages-plugin",
                "4slicer",
                "fixture-scenarios",
                "mongrel_proctitle",
                "rails-plugins",
            ],
        )

    def testGetLicenses(self):
        self.assertListKeyBegin(
            self.g.get_licenses(),
            lambda r: r.name,
            [
                "GNU General Public License v3.0",
                'BSD 2-Clause "Simplified" License',
                "MIT License",
                "GNU Lesser General Public License v2.1",
                "GNU General Public License v2.0",
                "GNU Lesser General Public License v3.0",
                "Mozilla Public License 2.0",
                'BSD 3-Clause "New" or "Revised" License',
            ],
        )

    def testGetLicense(self):
        self.assertEqual(
            self.g.get_license("mit").description,
            "A short and simple permissive license with conditions only requiring preservation of copyright and license notices. Licensed works, modifications, and larger works may be distributed under different terms and without source code.",
        )

    def testGetEvents(self):
        self.assertListKeyBegin(
            self.g.get_events(),
            lambda e: e.type,
            ["PushEvent", "WatchEvent", "PushEvent", "CommitCommentEvent"],
        )

    def testRequester(self):
        assert self.g.requester is self.g.__requester



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

from . import Framework


class Event(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.event = self.g.get_user("jacquev6").get_events()[0]

    def testAttributes(self):
        self.assertEqual(self.event.actor.login, "jacquev6")
        self.assertEqual(
            self.event.created_at,
            datetime(2012, 5, 26, 10, 1, 39, tzinfo=timezone.utc),
        )
        self.assertEqual(self.event.id, "1556114751")
        self.assertEqual(self.event.org, None)
        self.assertEqual(
            self.event.payload,
            {
                "commits": [
                    {
                        "url": "https://api.github.com/repos/jacquev6/PyGithub/commits/5bb654d26dd014d36794acd1e6ecf3736f12aad7",
                        "sha": "5bb654d26dd014d36794acd1e6ecf3736f12aad7",
                        "message": "Implement the three authentication schemes",
                        "distinct": False,
                        "author": {
                            "name": "Vincent Jacques",
                            "email": "vincent@vincent-jacques.net",
                        },
                    },
                    {
                        "url": "https://api.github.com/repos/jacquev6/PyGithub/commits/cb0313157bf904f2d364377d35d9397b269547a5",
                        "sha": "cb0313157bf904f2d364377d35d9397b269547a5",
                        "message": "Merge branch 'topic/Authentication' into develop",
                        "distinct": False,
                        "author": {
                            "name": "Vincent Jacques",
                            "email": "vincent@vincent-jacques.net",
                        },
                    },
                    {
                        "url": "https://api.github.com/repos/jacquev6/PyGithub/commits/0cec0d25e606c023a62a4fc7cdc815309ebf6d16",
                        "sha": "0cec0d25e606c023a62a4fc7cdc815309ebf6d16",
                        "message": "Publish version 0.7",
                        "distinct": False,
                        "author": {
                            "name": "Vincent Jacques",
                            "email": "vincent@vincent-jacques.net",
                        },
                    },
                    {
                        "url": "https://api.github.com/repos/jacquev6/PyGithub/commits/ecda065e01876209d2bdf5fe4e91cee8ffaa9ff7",
                        "sha": "ecda065e01876209d2bdf5fe4e91cee8ffaa9ff7",
                        "message": "Merge branch 'develop'",
                        "distinct": False,
                        "author": {
                            "name": "Vincent Jacques",
                            "email": "vincent@vincent-jacques.net",
                        },
                    },
                    {
                        "url": "https://api.github.com/repos/jacquev6/PyGithub/commits/3a3bf4763192ee1234eb0557628133e06f3dfc76",
                        "sha": "3a3bf4763192ee1234eb0557628133e06f3dfc76",
                        "message": "Merge branch 'master' into topic/RewriteWithGeneratedCode\n\nConflicts:\n\tgithub/Github.py\n\tgithub/Requester.py",
                        "distinct": True,
                        "author": {
                            "name": "Vincent Jacques",
                            "email": "vincent@vincent-jacques.net",
                        },
                    },
                    {
                        "url": "https://api.github.com/repos/jacquev6/PyGithub/commits/608f17794664f61693a3dc05e6056fea8fbef0ff",
                        "sha": "608f17794664f61693a3dc05e6056fea8fbef0ff",
                        "message": "Restore some form of Authorization header in replay data",
                        "distinct": True,
                        "author": {
                            "name": "Vincent Jacques",
                            "email": "vincent@vincent-jacques.net",
                        },
                    },
                    {
                        "url": "https://api.github.com/repos/jacquev6/PyGithub/commits/2c04b8adbd91d38eef4f0767337ab7a12b2f684b",
                        "sha": "2c04b8adbd91d38eef4f0767337ab7a12b2f684b",
                        "message": "Allow test without pre-set-up Github",
                        "distinct": True,
                        "author": {
                            "name": "Vincent Jacques",
                            "email": "vincent@vincent-jacques.net",
                        },
                    },
                    {
                        "url": "https://api.github.com/repos/jacquev6/PyGithub/commits/5b97389988b6fe43e15a079702f6f1671257fb28",
                        "sha": "5b97389988b6fe43e15a079702f6f1671257fb28",
                        "message": "Test three authentication schemes",
                        "distinct": True,
                        "author": {
                            "name": "Vincent Jacques",
                            "email": "vincent@vincent-jacques.net",
                        },
                    },
                    {
                        "url": "https://api.github.com/repos/jacquev6/PyGithub/commits/12747613c5ec00deccf296b8619ad507f7050475",
                        "sha": "12747613c5ec00deccf296b8619ad507f7050475",
                        "message": "Test Issue.getComments",
                        "distinct": True,
                        "author": {
                            "name": "Vincent Jacques",
                            "email": "vincent@vincent-jacques.net",
                        },
                    },
                    {
                        "url": "https://api.github.com/repos/jacquev6/PyGithub/commits/2982fa96c5ca75abe717d974d83f9135d664232e",
                        "sha": "2982fa96c5ca75abe717d974d83f9135d664232e",
                        "message": "Test the new Repository.full_name attribute",
                        "distinct": True,
                        "author": {
                            "name": "Vincent Jacques",
                            "email": "vincent@vincent-jacques.net",
                        },
                    },
                    {
                        "url": "https://api.github.com/repos/jacquev6/PyGithub/commits/619eae8d51c5988f0d2889fc767fa677438ba95d",
                        "sha": "619eae8d51c5988f0d2889fc767fa677438ba95d",
                        "message": "Improve coverage of AuthenticatedUser",
                        "distinct": True,
                        "author": {
                            "name": "Vincent Jacques",
                            "email": "vincent@vincent-jacques.net",
                        },
                    },
                ],
                "head": "619eae8d51c5988f0d2889fc767fa677438ba95d",
                "push_id": 80673538,
                "ref": "refs/heads/topic/RewriteWithGeneratedCode",
                "size": 11,
            },
        )
        self.assertTrue(self.event.public)
        self.assertEqual(self.event.repo.name, "jacquev6/PyGithub")
        self.assertEqual(self.event.type, "PushEvent")
        self.assertEqual(repr(self.event), 'Event(type="PushEvent", id="1556114751")')



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2019 Adam Baratz <adam.baratz@gmail.com>                           #
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Liuyang Wan <tsfdye@gmail.com>                                #
# Copyright 2020 Michał Górny <mgorny@gentoo.org>                              #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Maja Massarini <2678400+majamassarini@users.noreply.github.com>#
# Copyright 2026 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################


import itertools
from io import StringIO
from unittest.mock import Mock

import pytest
import responses

from . import Framework

PARAMETERS = itertools.product(
    [
        (Framework.ReplayingHttpConnection, "http"),
        (Framework.ReplayingHttpsConnection, "https"),
    ],
    [
        (
            '{"body":"BODY TEXT"}',
            "\nGET\napi.github.com\nNone\n/user\n{'Authorization': 'Basic login_and_password_removed', 'User-Agent': 'PyGithub/Python'}\nNone\n200\n[]\n{\"body\":\"BODY TEXT\"}\n\n",
        ),
        (
            '{"body":"BODY\xa0TEXT"}',
            "\nGET\napi.github.com\nNone\n/user\n{'Authorization': 'Basic login_and_password_removed', 'User-Agent': 'PyGithub/Python'}\nNone\n200\n[]\n{\"body\":\"BODY\xa0TEXT\"}\n\n",
        ),
        (
            "BODY TEXT",
            "\nGET\napi.github.com\nNone\n/user\n{'Authorization': 'Basic login_and_password_removed', 'User-Agent': 'PyGithub/Python'}\nNone\n200\n[]\nBODY TEXT\n\n",
        ),
        (
            "BODY\xa0TEXT",
            "\nGET\napi.github.com\nNone\n/user\n{'Authorization': 'Basic login_and_password_removed', 'User-Agent': 'PyGithub/Python'}\nNone\n200\n[]\nBODY\xa0TEXT\n\n",
        ),
    ],
)


class RecordingMockConnection(Framework.RecordingConnection):
    def __init__(self, protocol, host, port, realConnection):
        self._realConnection = realConnection
        super().__init__(protocol, host, port)


@pytest.mark.parametrize(
    ("replaying_connection_class", "protocol", "response_body", "expected_recording"),
    list(tuple(itertools.chain(*p)) for p in PARAMETERS),
)
@responses.activate
def testRecordAndReplay(replaying_connection_class, protocol, response_body, expected_recording):
    file = StringIO()
    host = "api.github.com"
    verb = "GET"
    url = "/user"
    headers = {"Authorization": "Basic p4ssw0rd", "User-Agent": "PyGithub/Python"}

    response = Mock()
    response.status = 200
    response.getheaders.return_value = {}
    response.read.return_value = response_body

    connection = Mock()
    connection.getresponse.return_value = response

    # write mock response to buffer
    rdf = Framework.ReplayDataFile("string", file)
    RecordingMockConnection.setOpenFile(lambda mode: rdf)
    recording_connection = RecordingMockConnection(protocol, host, None, lambda *args, **kwds: connection)
    recording_connection.request(verb, url, None, headers)
    recording_connection.getresponse()
    recording_connection.close()

    # validate contents of buffer
    file_value_lines = file.getvalue().split("\n")
    expected_recording_lines = (protocol + expected_recording).split("\n")
    assert file_value_lines[:5] == expected_recording_lines[:5]
    assert eval(file_value_lines[5]) == eval(expected_recording_lines[5])
    # dict literal, so keys not in guaranteed order
    assert file_value_lines[6:] == expected_recording_lines[6:]

    # rewind buffer and attempt to replay response from it
    file.seek(0)
    rdf = Framework.ReplayDataFile("string", file)
    replaying_connection_class.setOpenFile(lambda mode: rdf)
    replaying_connection = replaying_connection_class(host=host, port=None)
    replaying_connection.request(verb, url, None, headers)
    replaying_connection.getresponse()



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2020 Mahesh Raju <coder@mahesh.net>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2023 chantra <chantra@users.noreply.github.com>                    #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

import github
import github.Organization

from . import Framework
from .GithubIntegration import APP_ID, PRIVATE_KEY


class GithubApp(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.app_slug = "github-actions"
        self.app = self.g.get_app(slug=self.app_slug)
        # fetch lazy object
        self.app.id

    def testAttributes(self):
        app = self.app
        self.assertEqual(app.client_id, "Iv1.05c79e9ad1f6bdfa")
        self.assertIsNone(app.client_secret)
        self.assertEqual(app.created_at, datetime(2018, 7, 30, 9, 30, 17, tzinfo=timezone.utc))
        self.assertEqual(app.description, "Automate your workflow from idea to production")
        self.assertListEqual(
            app.events,
            [
                "branch_protection_rule",
                "check_run",
                "check_suite",
                "create",
                "delete",
                "deployment",
                "deployment_status",
                "discussion",
                "discussion_comment",
                "fork",
                "gollum",
                "issues",
                "issue_comment",
                "label",
                "merge_group",
                "milestone",
                "page_build",
                "project",
                "project_card",
                "project_column",
                "public",
                "pull_request",
                "pull_request_review",
                "pull_request_review_comment",
                "push",
                "registry_package",
                "release",
                "repository",
                "repository_dispatch",
                "status",
                "watch",
                "workflow_dispatch",
                "workflow_run",
            ],
        )
        self.assertEqual(app.external_url, "https://help.github.com/en/actions")
        self.assertEqual(app.html_url, "https://github.com/apps/github-actions")
        self.assertEqual(app.id, 15368)
        self.assertIsNone(app.installations_count)
        self.assertEqual(app.name, "GitHub Actions")
        self.assertEqual(app.node_id, "MDM6QXBwMTUzNjg=")
        self.assertIsInstance(app.owner, github.Organization.Organization)
        self.assertEqual(app.owner.login, "github")
        self.assertIsNone(app.pem)
        self.assertDictEqual(
            app.permissions,
            {
                "actions": "write",
                "administration": "read",
                "attestations": "write",
                "checks": "write",
                "contents": "write",
                "deployments": "write",
                "discussions": "write",
                "issues": "write",
                "merge_queues": "write",
                "metadata": "read",
                "packages": "write",
                "pages": "write",
                "pull_requests": "write",
                "repository_hooks": "write",
                "repository_projects": "write",
                "security_events": "write",
                "statuses": "write",
                "vulnerability_alerts": "read",
            },
        )
        self.assertEqual(app.slug, "github-actions")
        self.assertEqual(app.updated_at, datetime(2024, 4, 10, 20, 33, 16, tzinfo=timezone.utc))
        self.assertEqual(app.url, "/apps/github-actions")
        self.assertIsNone(app.webhook_secret)

    def testGetAuthenticatedApp(self):
        auth = github.Auth.AppAuth(APP_ID, PRIVATE_KEY)
        g = github.Github(auth=auth)

        with self.assertWarns(DeprecationWarning) as warning:
            # httpretty has some deprecation warnings in Python 3.12
            with self.ignoreWarning(category=DeprecationWarning, module="httpretty"):
                app = g.get_app()

            self.assertWarning(
                warning,
                "Argument slug is mandatory, calling this method without the slug argument is deprecated, "
                "please use github.GithubIntegration(auth=github.Auth.AppAuth(...)).get_app() instead",
            )

        self.assertEqual(app.created_at, datetime(2020, 8, 1, 17, 23, 46, tzinfo=timezone.utc))
        self.assertEqual(app.description, "Sample App to test PyGithub")
        self.assertListEqual(
            app.events,
            ["check_run", "check_suite", "label", "member", "public"],
        )
        self.assertEqual(app.external_url, "https://pygithub.readthedocs.io")
        self.assertEqual(app.html_url, "https://github.com/apps/pygithubtest")
        self.assertEqual(app.id, 75269)
        self.assertEqual(app.name, "PyGithubTest")
        self.assertEqual(app.owner.login, "wrecker")
        self.assertDictEqual(
            app.permissions,
            {
                "actions": "write",
                "checks": "write",
                "keys": "read",
                "members": "read",
                "metadata": "read",
                "packages": "read",
                "pages": "read",
                "repository_hooks": "write",
                "vulnerability_alerts": "read",
                "workflows": "write",
            },
        )
        self.assertEqual(app.slug, "pygithubtest")
        self.assertEqual(app.updated_at, datetime(2020, 8, 1, 17, 44, 31, tzinfo=timezone.utc))
        self.assertEqual(app.url, "/apps/pygithubtest")



#!/usr/bin/env python
############################ Copyrights and license ############################
#                                                                              #
# Copyright 2025 Changyong Um <e7217@naver.com>                                #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from . import Framework


class SubIssue(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.repo = self.g.get_repo("PyGithub/PyGithub", lazy=True)
        self.issue = self.repo.get_issue(5)

    def testListSubIssues(self):
        """
        Test listing sub-issues of an issue.
        """
        self.assertListKeyEqual(self.issue.get_sub_issues(), lambda s: s.number, [34, 35])

    def testAddSubIssue(self):
        """
        Test adding a sub-issue to an issue.
        """
        initial_sub_issues = list(self.issue.get_sub_issues())
        self.assertListKeyEqual(initial_sub_issues, lambda s: s.number, [34, 35, 38])

        sub_issue = self.repo.get_issue(39)
        self.issue.add_sub_issue(sub_issue)

        updated_sub_issues = list(self.issue.get_sub_issues())
        self.assertListKeyEqual(updated_sub_issues, lambda s: s.number, [34, 35, 38, 39])

    def testRemoveSubIssue(self):
        """
        Test removing a sub-issue from an issue.
        """
        initial_sub_issues = list(self.issue.get_sub_issues())
        self.assertListKeyEqual(initial_sub_issues, lambda s: s.number, [34, 35, 38, 39])

        sub_issue = self.repo.get_issue(39)
        self.issue.remove_sub_issue(sub_issue)

        updated_sub_issues = list(self.issue.get_sub_issues())
        self.assertListKeyEqual(updated_sub_issues, lambda s: s.number, [34, 35, 38])

    def testPrioritizeSubIssue(self):
        """
        Test changing the priority of a sub-issue.
        """
        initial_sub_issues = list(self.issue.get_sub_issues())
        self.assertListKeyEqual(initial_sub_issues, lambda s: s.number, [34, 35, 38])

        sub_issue = self.repo.get_issue(35)
        after_issue = self.repo.get_issue(38)
        self.issue.prioritize_sub_issue(sub_issue, after_issue)

        updated_sub_issues = list(self.issue.get_sub_issues())
        self.assertListKeyEqual(updated_sub_issues, lambda s: s.number, [34, 38, 35])



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2023 Andrew Dawes <53574062+AndrewJDawes@users.noreply.github.com> #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Hemslo Wang <hemslo.wang@gmail.com>                           #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

import inspect
import pickle
import sys
from abc import ABC

import github
from github.Auth import AppAuth, AppAuthToken, AppInstallationAuth, AppUserAuth, Auth, Login, NetrcAuth, Token
from github.PaginatedList import PaginatedList
from github.Repository import Repository

from . import Framework

REPO_NAME = "PyGithub/PyGithub"


class Pickle(Framework.TestCase):
    def testPickleGithub(self):
        gh = github.Github()
        gh2 = pickle.loads(pickle.dumps(gh))
        self.assertIsInstance(gh2, github.Github)
        self.assertIsNotNone(gh2._Github__requester._Requester__connection_lock)
        self.assertIsNone(gh2._Github__requester._Requester__connection)
        self.assertEqual(len(gh2._Github__requester._Requester__custom_connections), 0)

    def testPickleRepository(self):
        gh = github.Github(lazy=True)
        repo = gh.get_repo(REPO_NAME)
        repo2 = pickle.loads(pickle.dumps(repo))
        self.assertIsInstance(repo2, Repository)
        self.assertIsNotNone(repo2._requester._Requester__connection_lock)
        self.assertIsNone(repo2._requester._Requester__connection)
        self.assertEqual(len(repo2._requester._Requester__custom_connections), 0)

    def testPicklePaginatedList(self):
        gh = github.Github()
        repo = gh.get_repo(REPO_NAME, lazy=True)
        branches = repo.get_branches()
        branches2 = pickle.loads(pickle.dumps(branches))
        self.assertIsInstance(branches2, PaginatedList)

    auths = [
        Login("login", "password"),
        Token("token"),
        AppAuth("id", "key"),
        AppAuthToken("token"),
        AppInstallationAuth(AppAuth("id", "key"), 123),
        AppUserAuth("client_id", "client_secret", "access_token"),
        NetrcAuth(),
    ]

    def testPickleAuthSetup(self):
        # check we are testing *all* exiting auth classes
        auth_module = sys.modules[github.Auth.__name__]
        existing_auths = [
            clazz_type.__name__
            for clazz, clazz_type in inspect.getmembers(auth_module, inspect.isclass)
            if Auth in clazz_type.mro() and ABC not in clazz_type.__bases__
        ]
        tested_auths = [type(auth).__name__ for auth in self.auths]
        self.assertSequenceEqual(sorted(existing_auths), sorted(tested_auths))

    def testPickleAuth(self):
        for auth in self.auths:
            with self.subTest(auth=type(auth).__name__):
                auth2 = pickle.loads(pickle.dumps(auth))
                self.assertIsInstance(auth2, type(auth))



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2017 Simon <spam@esemi.ru>                                         #
# Copyright 2018 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from . import Framework


class ContentFile(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.file = self.g.get_repo("PyGithub/PyGithub").get_readme()

    def testAttributes(self):
        self.assertEqual(
            self.file._links,
            {
                "self": "https://api.github.com/repos/PyGithub/PyGithub/contents/README.md?ref=main",
                "git": "https://api.github.com/repos/PyGithub/PyGithub/git/blobs/0d9df5bfb7d4b93443c130bc8f4eea5dd3f01205",
                "html": "https://github.com/PyGithub/PyGithub/blob/main/README.md",
            },
        )
        self.assertIsNone(self.file.commit)
        self.assertTrue(self.file.content.startswith("IyBQeUdpdEh1YgoKWyFbUHlQSV0oaHR0cHM6Ly9p"))
        self.assertEqual(self.file.download_url, "https://raw.githubusercontent.com/PyGithub/PyGithub/main/README.md")
        self.assertEqual(self.file.encoding, "base64")
        self.assertIsNone(self.file.file_size)
        self.assertEqual(
            self.file.git_url,
            "https://api.github.com/repos/PyGithub/PyGithub/git/blobs/0d9df5bfb7d4b93443c130bc8f4eea5dd3f01205",
        )
        self.assertEqual(self.file.html_url, "https://github.com/PyGithub/PyGithub/blob/main/README.md")
        self.assertIsNone(self.file.language)
        self.assertIsNone(self.file.last_modified_at)
        self.assertIsNone(self.file.license)
        self.assertIsNone(self.file.line_numbers)
        self.assertEqual(self.file.name, "README.md")
        self.assertEqual(self.file.path, "README.md")
        self.assertEqual(self.file.sha, "0d9df5bfb7d4b93443c130bc8f4eea5dd3f01205")
        self.assertEqual(self.file.size, 2524)
        self.assertIsNone(self.file.submodule_git_url)
        self.assertIsNone(self.file.target)
        self.assertIsNone(self.file.text_matches)
        self.assertEqual(self.file.type, "file")
        self.assertEqual(self.file.encoding, "base64")
        self.assertEqual(self.file.size, 2524)
        self.assertEqual(self.file.name, "README.md")
        self.assertEqual(self.file.path, "README.md")
        self.assertEqual(len(self.file.content), 3425)
        self.assertEqual(len(self.file.decoded_content), 2524)
        self.assertEqual(self.file.sha, "0d9df5bfb7d4b93443c130bc8f4eea5dd3f01205")
        self.assertEqual(self.file.download_url, "https://raw.githubusercontent.com/PyGithub/PyGithub/main/README.md")
        self.assertIsNone(self.file.license)
        self.assertEqual(repr(self.file), 'ContentFile(path="README.md")')
        self.assertEqual(self.file.url, "https://api.github.com/repos/PyGithub/PyGithub/contents/README.md?ref=main")
